@@ -2,7 +2,7 @@ import os
 
 __pretty_app_name__ = "Gonvert"
 __app_name__ = "gonvert"
-__version__ = "1.1.3"
+__version__ = "1.1.4"
 __build__ = 0
 __app_magic__ = 0xdeadbeef
 _data_path_ = os.path.join(os.path.expanduser("~"), ".gonvert")
@@ -10,4 +10,4 @@ _user_settings_ = "%s/settings.json" % _data_path_
 _user_logpath_ = "%s/gonvert.log" % _data_path_
 
 PROFILE_STARTUP = False
-FORCE_HILDON_LIKE = False
+IS_MAEMO = True
