@@ -5,8 +5,8 @@ import sys
 import logging
 
 
-_moduleLogger = logging.getLogger("gonvert")
-sys.path.append("/usr/lib/gonvert/")
+_moduleLogger = logging.getLogger(__name__)
+sys.path.append("/opt/gonvert/lib")
 
 
 import constants
